@@ -101,7 +101,7 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
     }
 
     private static final String
-    STRINGIO_VERSION = "3.1.7";
+    STRINGIO_VERSION = "3.1.8";
 
     private static final int STRIO_READABLE = ObjectFlags.registry.newFlag(StringIO.class);
     private static final int STRIO_WRITABLE = ObjectFlags.registry.newFlag(StringIO.class);
@@ -149,7 +149,7 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
         }
 
         RubyString string = ptr.string;
-        if (string != null && !string.isNil()) {
+        if (string != null) {
             return string.getEncoding();
         }
 
@@ -650,13 +650,17 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
     public IRubyObject eof(ThreadContext context) {
         checkReadable();
         StringIOData ptr = getPtr();
-        if (ptr.pos < ptr.string.size()) return context.fals;
+        if (!isEndOfString()) return context.fals;
         return context.tru;
     }
 
     private boolean isEndOfString() {
+        return isOutside(ptr.pos);
+    }
+
+    private boolean isOutside(int pos) {
         StringIOData ptr = getPtr();
-        return ptr.string == null || ptr.pos >= ptr.string.size();
+        return ptr.string == null || pos >= ptr.string.size();
     }
 
     @JRubyMethod(name = "getc")
@@ -1063,19 +1067,19 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
 
     public static final ByteList NEWLINE = ByteList.create("\n");
 
-    @JRubyMethod(name = "read")
+    @JRubyMethod(name = "read") // strio.read()
     public IRubyObject read(ThreadContext context) {
         return readCommon(context, 0, null, null);
     }
 
-    @JRubyMethod(name = "read")
-    public IRubyObject read(ThreadContext context, IRubyObject arg0) {
-        return readCommon(context, 1, arg0, null);
+    @JRubyMethod(name = "read") // strio.read(length)
+    public IRubyObject read(ThreadContext context, IRubyObject length) {
+        return readCommon(context, 1, length, null);
     }
 
-    @JRubyMethod(name = "read")
-    public IRubyObject read(ThreadContext context, IRubyObject arg0, IRubyObject arg1) {
-        return readCommon(context, 2, arg0, arg1);
+    @JRubyMethod(name = "read") // strio.read(length, outbuf)
+    public IRubyObject read(ThreadContext context, IRubyObject length, IRubyObject outbuf) {
+        return readCommon(context, 2, length, outbuf);
     }
 
     @SuppressWarnings("fallthrough")
@@ -1098,7 +1102,7 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
                     str = arg1;
                     if (!str.isNil()) {
                         str = str.convertToString();
-                        ((RubyString) str).modify();
+                        modifyString((RubyString) str);
                     }
                 case 1:
                     if (!arg0.isNil()) {
@@ -1107,9 +1111,9 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
                         if (len < 0) {
                             throw runtime.newArgumentError("negative length " + len + " given");
                         }
-                        if (len > 0 && isEndOfString()) {
+                        if (isEndOfString()) {
                             if (!str.isNil()) ((RubyString) str).resize(0);
-                            return context.nil;
+                            return len > 0 ? context.nil : runtime.newString();
                         }
                         binary = true;
                         break;
@@ -1194,7 +1198,7 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
                 str = arg2;
                 if (!str.isNil()) {
                     str = str.convertToString();
-                    ((RubyString) str).modify();
+                    modifyString((RubyString) str);
                 }
             case 2:
                 len = RubyNumeric.fix2int(arg0);
@@ -1224,8 +1228,7 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
                 throw runtime.newErrnoEINVALError("pread: Invalid offset argument");
             }
 
-            RubyString myString = ptr.string;
-            if (offset >= myString.size()) {
+            if (isOutside(offset)) {
                 throw context.runtime.newEOFError();
             }
 
@@ -1234,6 +1237,7 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
             }
 
             string = (RubyString) str;
+            RubyString myString = ptr.string;
             int rest = myString.size() - offset;
             if (len > rest) len = rest;
             string.resize(len);
@@ -1378,7 +1382,11 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
                     offset += ptr.pos;
                     break;
                 case 2:
-                    offset += ptr.string.size();
+                    if (ptr.string == null) {
+                        offset += 0;
+                    } else {
+                        offset += ptr.string.size();
+                    }
                     break;
                 default:
                     throw runtime.newErrnoEINVALError("invalid whence");
@@ -2202,12 +2210,13 @@ public class StringIO extends RubyObject implements EncodingCapable, DataType {
     }
 
     private void checkModifiable() {
-        if (getPtr().string == null || getPtr().string.isNil()) {
+        final RubyString string = getPtr().string;
+        if (string == null) {
             /* Null device StringIO */
-        } else if (getPtr().string.isFrozen()) {
+        } else if (string.isFrozen()) {
             throw getRuntime().newIOError("not modifiable string");
         } else {
-            getPtr().string.modify();
+            string.modify();
         }
     }
 
